﻿using CH09_TcpServer;

using System;
using System.Buffers;
using System.IO.Pipelines;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;


Socket listenSocket = new Socket(SocketType.Stream, ProtocolType.Tcp);
listenSocket.Bind(new IPEndPoint(IPAddress.Loopback, 7000));

Console.WriteLine("Listening on port 7000");

listenSocket.Listen(120);

while (true)
{
    Socket socket = await listenSocket.AcceptAsync();
    _ = ProcessLinesAsync(socket);
}


static async Task ProcessLinesAsync(Socket socket)
{
    Console.WriteLine($"[{socket.RemoteEndPoint}]: connected");

		NetworkStream stream = new NetworkStream(socket);
		PipeReader reader = PipeReader.Create(stream);

    while (true)
    {
        ReadResult result = await reader.ReadAsync();
        ReadOnlySequence<byte> buffer = result.Buffer;

        while (TryReadLine(ref buffer, out ReadOnlySequence<byte> line))
            ProcessLine(line);

        reader.AdvanceTo(buffer.Start, buffer.End);

        if (result.IsCompleted)
            break;
    }

    await reader.CompleteAsync();

    Console.WriteLine($"[{socket.RemoteEndPoint}]: disconnected");
}

static bool TryReadLine(ref ReadOnlySequence<byte> buffer, out ReadOnlySequence<byte> line)
{
    SequencePosition? position = buffer.PositionOf((byte)'\n');

    if (position == null)
    {
        line = default;
        return false;
    }

    line = buffer.Slice(0, position.Value);
    buffer = buffer.Slice(buffer.GetPosition(1, position.Value));
    return true;
}

static void ProcessLine(in ReadOnlySequence<byte> buffer)
{
    foreach (ReadOnlyMemory<byte> segment in buffer)
    {
        Console.Write(Encoding.UTF8.GetString(segment.Span));
    }
    Console.WriteLine();
}
