# High-Performance Programming in C# and .NET 

<a href="https://www.packtpub.com/product/high-performance-programming-in-c-and-net/9781800564718?utm_source=github&utm_medium=repository&utm_campaign="><img src="https://static.packt-cdn.com/products/9781800564718/cover/smaller" alt="High-Performance Programming in C# and .NET " height="256px" align="right"></a>

This is the code repository for [High-Performance Programming in C# and .NET ](https://www.packtpub.com/product/high-performance-programming-in-c-and-net/9781800564718?utm_source=github&utm_medium=repository&utm_campaign=), published by Packt.

**Understand the nuts and bolts of developing robust, faster, and resilient applications in C# 10.0 and .NET 6**

## What is this book about?
This book provides comprehensive guidance on how to design high-performance software using practical examples to help you understand the key factors involved in boosting application performance. By the end of this book, you’ll have gained a competitive advantage by thoroughly understanding the key aspects of writing highly efficient software.

This book covers the following exciting features:
* Use correct types and collections to enhance application performance
* Profile, benchmark, and identify performance issues with the codebase
* Explore how to best perform queries on LINQ to improve an application's performance
* Effectively utilize a number of CPUs and cores through asynchronous programming
* Build responsive user interfaces with WinForms, WPF, MAUI, and WinUI
* Benchmark ADO.NET, Entity Framework Core, and Dapper for data access
* Implement CQRS and event sourcing and build and deploy microservices

If you feel this book is for you, get your [copy](https://www.amazon.com/dp/1800564716) today!

<a href="https://www.packtpub.com/?utm_source=github&utm_medium=banner&utm_campaign=GitHubBanner"><img src="https://raw.githubusercontent.com/PacktPublishing/GitHub/master/GitHub.png" 
alt="https://www.packtpub.com/" border="5" /></a>

## Instructions and Navigations
All of the code is organized into folders. For example, Chapter02.

The code will look like the following:
```
static void Main(string[] _)
{
Console.WriteLine(Chapter 3: Strings are immutable);
var greeting1 = Hello, world!;
var greeting2 = greeting1;
Console.WriteLine($greeting1={greeting1});
Console.WriteLine($greeting2={greeting2});
greeting1 += Isnt life grand!;
Console.WriteLine($greeting1={greeting1});
Console.WriteLine($greeting1={greeting2});
}
```

**Following is what you need for this book:**
This book is for software engineers, professional software developers, performance engineers, and application profilers looking to improve the speed of their code or take their skills to the next level to gain a competitive advantage. You should be a proficient C# programmer who can already put the language to good use and is also comfortable using Microsoft Visual Studio 2022.

With the following software and hardware list you can run all code files present in the book (Chapter 1-16).
### Software and Hardware List
| Chapter | Software required | OS required |
| -------- | ------------------------------------ | ----------------------------------- |
| 1 | .NET 6 SDK | Windows, Mac OS X, and Linux (Any) |
| 1 | Visual Studio 2022 | Windows, Mac OS X, and Linux (Any) |
| 10 | SQL Server | Windows, Mac OS X, and Linux (Any) |
| 10 | SQL Server Managment Srudio | Windows, Mac OS X, and Linux (Any) |


We also provide a PDF file that has color images of the screenshots/diagrams used in this book. [Click here to download it](https://packt.link/hQmsb).

### Related products
* C# 10 and .NET 6 Modern Cross-Platform Development - Sixth Edition  [[Packt]](https://www.packtpub.com/product/c-10-and-net-6-modern-cross-platform-development-sixth-edition/9781801077361?utm_source=github&utm_medium=repository&utm_campaign=) [[Amazon]](https://www.amazon.com/dp/1801077363)

* Software Architecture with C# 10 and .NET 6 - Third Edition  [[Packt]](https://www.packtpub.com/product/software-architecture-with-c-10-and-net-6-third-edition/9781803235257?utm_source=github&utm_medium=repository&utm_campaign=) [[Amazon]](https://www.amazon.com/dp/180323525X)

* Enterprise Application Development with C# 10 and .NET 6 - Second Edition  [[Packt]](https://www.packtpub.com/product/enterprise-application-development-with-c-10-and-net-6/9781803232973?utm_source=github&utm_medium=repository&utm_campaign=) [[Amazon]](https://www.amazon.com/dp/1803232978)



## Get to Know the Author
**Jason Alls**
is the author of Clean Code in C# and has been programming for over 21 years. Working with an Australasian company, he started his career developing call center management reporting software used by global clients including telecom providers, banks, airlines, and the police. He then moved on to develop GIS marketing applications and worked in the banking sector performing data migrations between Oracle and SQL Server. Certified as an MCAD in C# since 2005, he has been involved in the development of various desktop, web, and mobile applications.
Currently employed by a leading software house, he develops and supports order processing and warehouse management software written in C#.



## Other books by the authors
[Clean Code in C# ](https://www.packtpub.com/product/clean-code-in-c/9781838982973?utm_source=github&utm_medium=repository&utm_campaign=)

### Download a free PDF

 <i>If you have already purchased a print or Kindle version of this book, you can get a DRM-free PDF version at no cost.<br>Simply click on the link to claim your free PDF.</i>
<p align="center"> <a href="https://packt.link/free-ebook/9781800564718">https://packt.link/free-ebook/9781800564718 </a> </p>